﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.DragDrop;

namespace DragDropRecordsInHierarchy
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            Data = new ObservableCollection<Parent>
            {
                new Parent
                {
                    ID = 1,
                    Name = "Parent 1",
                    Children = new ObservableCollection<Child>
                    {
                        new Child { Age = 15, Name = "Child 1", ParentName = "Parnet 1" },
                        new Child { Age = 11, Name = "Child 2", ParentName = "Parnet 1" },
                        new Child { Age = 2, Name = "Child 3", ParentName = "Parnet 1" },
                    }
                },
                new Parent
                {
                    ID = 2,
                    Name = "Parent 2",
                    Children = new ObservableCollection<Child>
                    {
                        new Child { Age = 7, Name = "Child 1", ParentName = "Parnet 2" },
                        new Child { Age = 1, Name = "Child 2", ParentName = "Parnet 2" },
                        new Child { Age = 4, Name = "Child 3", ParentName = "Parnet 2" },
                    }
                },
                new Parent
                {
                    ID = 3,
                    Name = "Parent 3",
                    Children = new ObservableCollection<Child>
                    {
                        new Child { Age = 5, Name = "Child 1", ParentName = "Parnet 3" },
                        new Child { Age = 17, Name = "Child 2", ParentName = "Parnet 3" },
                        new Child { Age = 18, Name = "Child 3", ParentName = "Parnet 3" },
                    }
                }
            };

            DataContext = Data;
        }

        public ObservableCollection<Parent> Data { get; set; }

        private void PART_RecordContentSite_Loaded(object sender, RoutedEventArgs e)
        {
            if ((sender as ContentPresenter).DataContext != null)
            {
                DragDropManager.GetDragSource(sender as DependencyObject).DragChannels = new ObservableCollection<string> { ((sender as ContentPresenter).DataContext as DataRecord).NestingDepth.ToString() };

                if (((sender as ContentPresenter).DataContext as DataRecord).RecordManager.ParentDataRecord == null)
                    DragDropManager.GetDragSource(sender as DependencyObject).DataObject = ((sender as ContentPresenter).DataContext as DataRecord).RecordManager.DataPresenter.DataSource;

                DragDropManager.GetDropTarget(sender as DependencyObject).DropChannels = new ObservableCollection<string> { ((sender as ContentPresenter).DataContext as DataRecord).NestingDepth.ToString() };
            }
        }

        private void DragSource_Drop(object sender, DropEventArgs e)
        {            
            if (e.Data is ObservableCollection<Parent>)
            {
                Parent drag = ((e.DragSource as ContentPresenter).DataContext as DataRecord).DataItem as Parent;
                Parent drop = ((e.DropTarget as ContentPresenter).DataContext as DataRecord).DataItem as Parent;

                ObservableCollection<Parent> parentCollection = e.Data as ObservableCollection<Parent>;

                int dragIndex = parentCollection.IndexOf(drag);
                int dropIndex = parentCollection.IndexOf(drop);

                parentCollection.Remove(drag);
                parentCollection.Insert(dropIndex, drag);
            }
            else
            {

                Child drag = ((e.DragSource as ContentPresenter).DataContext as DataRecord).DataItem as Child;
                Child drop = ((e.DropTarget as ContentPresenter).DataContext as DataRecord).DataItem as Child;

                ObservableCollection<Child> parentDragCollection = (((e.DragSource as ContentPresenter).DataContext as DataRecord).ParentDataRecord.DataItem as Parent).Children;
                ObservableCollection<Child> parentDropCollection = (((e.DropTarget as ContentPresenter).DataContext as DataRecord).ParentDataRecord.DataItem as Parent).Children;

                int dropIndex = parentDropCollection.IndexOf(drop);

                parentDragCollection.Remove(drag);
                parentDropCollection.Insert(dropIndex, drag);
            }
        }

        private void XamDataGrid_Loaded(object sender, RoutedEventArgs e)
        {
            (sender as XamDataGrid).Resources.Add(typeof(DataRecordPresenter), root.Resources["style"]);
        }
    }


    public class Parent
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public ObservableCollection<Child> Children { get; set; }
    }

    public class Child
    {
        public string Name { get; set; }
        public int Age { get; set; }
        public string ParentName { get; set; }
    }

    public class DepthToDragDropChannels : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return new ObservableCollection<string> { value.ToString() };
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
